import { Coord } from "@turf/helpers";
import { IMixParams } from "./types";
import { IDecoratedAsset } from "./types/asset";
export interface IAssetPriorities {
    readonly DISCARD: false;
    readonly NEUTRAL: number;
    readonly LOWEST: number;
    readonly NORMAL: number;
    readonly HIGHEST: number;
    readonly [key: string]: false | number;
}
export declare type AssetPriorityType = false | 0 | 1 | 100 | 999 | number;
export declare const ASSET_PRIORITIES: IAssetPriorities;
/**
 *Accept an asset if any one of the provided filters passes, returns the first non-discarded and non-neutral rank
 *
 * @export
 * @param {(Array<(asset: IDecoratedAsset, param?: IMixParams | any) => number>)} [filters=[]]
 * @param {IMixParams} [mixParams]
 * @return {*}
 */
export declare const anyAssetFilter: (filters: ((asset: IDecoratedAsset, param: IMixParams) => AssetPriorityType)[]) => (asset: IDecoratedAsset, { ...stateParams }: {
    [x: string]: any;
}) => number | false;
/** Filter composed of multiple inner filters that accepts assets which pass every inner filter. */
export declare function allAssetFilter(filters: ((asset: IDecoratedAsset, param: IMixParams) => AssetPriorityType)[], mixParams?: IMixParams): (asset: IDecoratedAsset, stateParams: IMixParams) => AssetPriorityType;
export declare const rankForGeofilteringEligibility: (asset: IDecoratedAsset | undefined, { listenerPoint, geoListenMode, }: Pick<IMixParams, "listenerPoint" | "geoListenMode">) => boolean;
export declare const calculateDistanceInMeters: (loc1: Coord, loc2: Coord) => number;
/** Only accepts an asset if the user is within the project-configured recording radius  */
export declare const distanceFixedFilter: () => (asset: IDecoratedAsset, options: Pick<IMixParams, "geoListenMode" | "listenerPoint" | "recordingRadius">) => AssetPriorityType;
/**
 Accepts an asset if the user is within range of it based on the current dynamic distance range.
 */
export declare const distanceRangesFilter: () => (asset: IDecoratedAsset, options: Pick<IMixParams, "getListenMode" | "listenerPoint" | "minDist" | "maxDist">) => number | false;
export declare function anyTagsFilter(): (asset: IDecoratedAsset, mixParams: IMixParams) => number | false;
export declare const timedAssetFilter: () => (asset: IDecoratedAsset, { elapsedSeconds, timedAssetPriority }: {
    elapsedSeconds?: number | undefined;
    timedAssetPriority?: string | undefined;
}) => number | false;
export declare const assetShapeFilter: () => (asset: IDecoratedAsset, options: Pick<IMixParams, "listenerPoint" | "geoListenMode">) => number | false;
export declare const timedRepeatFilter: () => (asset: IDecoratedAsset, { bannedDuration, repeatRecordings }: IMixParams) => number | false;
export declare const dateRangeFilter: () => (asset: IDecoratedAsset, { startDate, endDate }: {
    startDate?: Date | undefined;
    endDate?: Date | undefined;
}) => number | false;
export declare const pausedAssetFilter: () => (asset: IDecoratedAsset, mixParams?: IMixParams) => number | false;
export declare const roundwareDefaultFilterChain: (asset: IDecoratedAsset, mixParams: IMixParams) => AssetPriorityType;
