"use strict";

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.array.filter.js");

require("core-js/modules/es.object.get-own-property-descriptor.js");

require("core-js/modules/es.object.get-own-property-descriptors.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.assetDecorationMapper = exports.AssetPool = void 0;

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.array.map.js");

require("core-js/modules/web.dom-collections.for-each.js");

require("core-js/modules/es.array.includes.js");

require("core-js/modules/es.string.includes.js");

require("core-js/modules/es.object.keys.js");

require("core-js/modules/es.number.parse-int.js");

require("core-js/modules/es.number.constructor.js");

require("core-js/modules/es.array.sort.js");

var _lodash = require("lodash");

var _assetFilters = require("./assetFilters");

var _assetSorter = require("./assetSorter");

var _app = require("./errors/app.errors");

var _utils = require("./utils");

var _excluded = ["elapsedSeconds", "filterOutAssets", "listenTagIds"];

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// add new fields to assets after they have been downloaded from the API to be used by rest of the mixing code
// also rewrite .wav as .mp3
var assetDecorationMapper = function assetDecorationMapper(timedAssets) {
  var timedAssetLookup = timedAssets.reduce(function (lookupTable, timedAsset) {
    return _objectSpread(_objectSpread({}, lookupTable), {}, _defineProperty({}, timedAsset.asset_id, timedAsset));
  }, {});
  return function (asset) {
    var _asset$start_time = asset.start_time,
        activeRegionLowerBound = _asset$start_time === void 0 ? 0 : _asset$start_time,
        _asset$end_time = asset.end_time,
        activeRegionUpperBound = _asset$end_time === void 0 ? 0 : _asset$end_time,
        assetUrl = asset.file;
    var activeRegionLength = activeRegionUpperBound - activeRegionLowerBound; // per Halsey we should always use mp3s; also we avoid specifying http/https to avoid mixed-content warnings

    if (!assetUrl) console.warn("assetUrl was undefined!");
    var mp3Url = (0, _utils.cleanAudioURL)(assetUrl);

    var decoratedAsset = _objectSpread(_objectSpread({
      locationPoint: (0, _utils.coordsToPoints)({
        latitude: asset.latitude,
        longitude: asset.longitude
      }),
      playCount: 0,
      activeRegionLength: activeRegionLength,
      activeRegionUpperBound: activeRegionUpperBound,
      activeRegionLowerBound: activeRegionLowerBound
    }, asset), {}, {
      created: asset.created ? new Date(asset.created) : new Date(),
      file: mp3Url
    });

    var timedAsset = timedAssetLookup[asset.id];

    if (timedAsset) {
      decoratedAsset.timedAssetStart = timedAsset.start;
      decoratedAsset.timedAssetEnd = timedAsset.end;
    }

    return decoratedAsset;
  };
};

exports.assetDecorationMapper = assetDecorationMapper;

var AssetPool = /*#__PURE__*/function () {
  function AssetPool(_ref) {
    var _ref$assets = _ref.assets,
        assets = _ref$assets === void 0 ? [] : _ref$assets,
        _ref$timedAssets = _ref.timedAssets,
        timedAssets = _ref$timedAssets === void 0 ? [] : _ref$timedAssets,
        _ref$filterChain = _ref.filterChain,
        filterChain = _ref$filterChain === void 0 ? _assetFilters.roundwareDefaultFilterChain : _ref$filterChain,
        _ref$sortMethods = _ref.sortMethods,
        sortMethods = _ref$sortMethods === void 0 ? [] : _ref$sortMethods,
        _ref$mixParams = _ref.mixParams,
        mixParams = _ref$mixParams === void 0 ? {} : _ref$mixParams;

    _classCallCheck(this, AssetPool);

    _defineProperty(this, "assets", []);

    if (!Array.isArray(assets)) throw new _app.InvalidArgumentError("assets", "array of IAssetData", "instantiation assetPool");
    if (!Array.isArray(timedAssets)) throw new _app.InvalidArgumentError("timedAssets", "array of ITimedAssetData", "instantiation assetPool");
    this.updateAssets(assets, timedAssets);
    this.assetSorter = new _assetSorter.AssetSorter(_objectSpread({
      sortMethods: sortMethods
    }, mixParams));
    this.playingTracks = {};
    this.mixParams = mixParams;
    this.filterChain = filterChain;
    this.sortAssets();
  }

  _createClass(AssetPool, [{
    key: "updateAssets",
    value: function updateAssets() {
      var _this = this;

      var assets = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
      var timedAssets = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

      if (!Array.isArray(assets) || !Array.isArray(timedAssets)) {
        throw new _app.InvalidArgumentError("assets/timedAssets", "array", "updateAssets() in AssetPool");
      }

      var newAssets = assets.map(assetDecorationMapper(timedAssets)); // preserve the existing properties of assets, add instead of replacing...

      if (Array.isArray(this.assets)) {
        newAssets.forEach(function (asset) {
          if (!_this.assets.some(function (a) {
            return a.id === asset.id;
          })) _this.add(asset);
        });
      } else this.assets = newAssets;
    }
  }, {
    key: "nextForTrack",
    value: function nextForTrack(track, _ref2) {
      var _track$mixParams,
          _this2 = this;

      var elapsedSeconds = _ref2.elapsedSeconds,
          _ref2$filterOutAssets = _ref2.filterOutAssets,
          filterOutAssets = _ref2$filterOutAssets === void 0 ? [] : _ref2$filterOutAssets,
          _ref2$listenTagIds = _ref2.listenTagIds,
          listenTagIds = _ref2$listenTagIds === void 0 ? [] : _ref2$listenTagIds,
          stateParams = _objectWithoutProperties(_ref2, _excluded);

      var mixParams = _objectSpread(_objectSpread(_objectSpread(_objectSpread({}, this.mixParams), track.mixParams), stateParams), {}, {
        listenTagIds: (0, _lodash.union)(this.mixParams.listenTagIds, (_track$mixParams = track.mixParams) === null || _track$mixParams === void 0 ? void 0 : _track$mixParams.listenTagIds, listenTagIds)
      }, track.trackOptions);

      var rankedAssets = this.assets.reduce(function (rankings, asset) {
        if (filterOutAssets.includes(asset)) return rankings;

        var rank = _this2.filterChain(asset, mixParams);

        if (rank !== false) {
          rankings[rank] = rankings[rank] || [];
          rankings[rank].push(asset);
        }

        return rankings;
      }, {});
      var rankingGroups = Object.keys(rankedAssets).map(function (a) {
        return Number.parseInt(a);
      });

      if (Array.isArray(rankingGroups) && rankingGroups.length < 1) {
        console.warn("All assets filtered out");
        return;
      }

      var topPriorityRanking = rankingGroups.sort()[0]; // play least-recently played assets first

      var priorityAssets = rankedAssets[topPriorityRanking] || []; // this.assetSorter.sort(priorityAssets);

      priorityAssets.sort(function (a, b) {
        return b.playCount - a.playCount;
      });
      var nextAsset = priorityAssets.pop(); // if new asset is not an asset which was paused previously
      // then release all the paused assets

      if (!mixParams.keepPausedAssets) {
        if ((nextAsset === null || nextAsset === void 0 ? void 0 : nextAsset.status) !== "resumed") {
          this.assets.forEach(function (a) {
            if (a.status === "paused" && a.pausedFromTrackId === track.trackId) {
              track.pausedAssetId = null;
              a.status = undefined;
              a.playCount++;
            }
          });
        }
      }

      return nextAsset;
    }
  }, {
    key: "sortAssets",
    value: function sortAssets() {
      this.assetSorter.sort(this.assets);
    }
  }, {
    key: "add",
    value: function add(asset) {
      this.assets.push(asset);
      this.sortAssets();
    }
  }]);

  return AssetPool;
}();

exports.AssetPool = AssetPool;
var _default = AssetPool;
exports.default = _default;