import { ApiClient } from "./api-client";
import { IAssetData } from "./types/asset";
import { EventPayload, EventType } from "./types/events";
/**
 *
 * Post events to roundware server
 * @export
 * @class RoundwareEvents
 */
export declare class RoundwareEvents {
    _sessionId: number;
    _apiClient: ApiClient;
    _startedAssets: {
        [id: number]: {
            startTime: Date;
            id: number;
        };
    };
    /**
     * Creates an instance of RoundwareEvents.
     * @param {number} sessionId
     * @param {ApiClient} apiClient
     * @memberof RoundwareEvents
     */
    constructor(sessionId: number, apiClient: ApiClient);
    /**
     *
     * When any asset starts playing
     * @param {number} assetId -
     * @param {Date} [startTime = new Date()] (Optional) time when asset started playing, default to current time
     * @return {Promise} Promise of Response
     * @memberof RoundwareEvents
     */
    logAssetStart(assetId: IAssetData[`id`], startTime?: Date): Promise<void>;
    /**
     *
     * Calculates duration in seconds from when started, and sends patch request to givena assetId
     * @param {number} assetId - id of the asset
     * @return {Promise} Promise of Response
     * @memberof RoundwareEvents
     */
    logAssetEnd(assetId: IAssetData[`id`]): Promise<void>;
    logEvent(eventType: EventType, payload?: EventPayload): Promise<EventPayload>;
}
