import { IAudioContext } from "standardized-audio-context";
import { AssetPool } from "./assetPool";
import { Playlist } from "./playlist";
import { AssetPriorityType, Roundware } from "./roundware";
import { SpeakerTrack } from "./speaker_track";
import { Coordinates, GeoListenModeType, IMixParams } from "./types";
import { IDecoratedAsset } from "./types/asset";
export declare const GeoListenMode: {
    DISABLED: GeoListenModeType;
    MANUAL: GeoListenModeType;
    AUTOMATIC: GeoListenModeType;
};
export declare class Mixer {
    playing: boolean;
    private _windowScope;
    private _client;
    private _prefetchSpeakerAudio;
    mixParams: IMixParams;
    playlist: Playlist | undefined;
    assetPool: AssetPool;
    speakerTracks: SpeakerTrack[] | undefined;
    audioContext: IAudioContext;
    constructor({ client, windowScope, listenerLocation, filters, sortMethods, mixParams, }: {
        client: Roundware;
        windowScope: Window;
        listenerLocation: Coordinates;
        filters?: (asset: IDecoratedAsset, mixParams: IMixParams) => AssetPriorityType;
        sortMethods?: string[];
        mixParams: IMixParams;
    });
    updateParams({ listenerLocation, ...params }: IMixParams): void;
    /**
     * @param  {number} trackId
     */
    skipTrack(trackId: number): void;
    skip(): void;
    /**
     * @param  {number} trackId
     */
    replayTrack(trackId: number): void;
    /**
     * @returns string
     */
    toString(): string;
    /**
     * Builds speaker tracks & playlist instance if it doesn't exist yet
     */
    initContext(): void;
    /**
     * @param  {boolean} true to play false to stop
     * @returns boolean
     */
    toggle(play?: boolean): boolean;
    play(): void;
    stop(): void;
    endedSpeakersLength: number;
    handleSpeakerEnd(): void;
    replay(): void;
    initializeSpeakers(): void;
    allSpeakersEndCallback: () => void;
    onAllSpeakersEnd(callback: () => void): void;
}
