"use strict";

require("core-js/modules/es.object.keys.js");

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.object.get-own-property-descriptor.js");

require("core-js/modules/es.object.get-own-property-descriptors.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Mixer = exports.GeoListenMode = void 0;

require("core-js/modules/es.object.freeze.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/web.dom-collections.for-each.js");

require("core-js/modules/es.regexp.to-string.js");

require("core-js/modules/es.number.constructor.js");

require("core-js/modules/es.array.filter.js");

require("core-js/modules/es.array.map.js");

var _assetPool = require("./assetPool");

var _playlist = require("./playlist");

var _speaker_track = require("./speaker_track");

var _utils = require("./utils");

var _excluded = ["listenerLocation"];

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var GeoListenMode = Object.freeze({
  DISABLED: 0,
  MANUAL: 1,
  AUTOMATIC: 2
});
exports.GeoListenMode = GeoListenMode;

var Mixer = /*#__PURE__*/function () {
  function Mixer(_ref) {
    var client = _ref.client,
        windowScope = _ref.windowScope,
        listenerLocation = _ref.listenerLocation,
        filters = _ref.filters,
        _ref$sortMethods = _ref.sortMethods,
        sortMethods = _ref$sortMethods === void 0 ? [] : _ref$sortMethods,
        _ref$mixParams = _ref.mixParams,
        mixParams = _ref$mixParams === void 0 ? {} : _ref$mixParams;

    _classCallCheck(this, Mixer);

    _defineProperty(this, "endedSpeakersLength", 0);

    _defineProperty(this, "allSpeakersEndCallback", function () {});

    this.playing = false;
    this._windowScope = windowScope;
    this._client = client;
    var assets = client.assets();
    var timedAssets = client.timedAssets();
    this.mixParams = _objectSpread({
      listenerPoint: (0, _utils.coordsToPoints)({
        latitude: listenerLocation.latitude,
        longitude: listenerLocation.longitude
      })
    }, mixParams);
    this.assetPool = new _assetPool.AssetPool({
      assets: assets,
      timedAssets: timedAssets,
      filterChain: filters,
      sortMethods: sortMethods,
      mixParams: this.mixParams
    });
    this.audioContext = (0, _utils.buildAudioContext)(this._windowScope);
  }

  _createClass(Mixer, [{
    key: "updateParams",
    value: function updateParams(_ref2) {
      var _this = this;

      var listenerLocation = _ref2.listenerLocation,
          params = _objectWithoutProperties(_ref2, _excluded);

      if (listenerLocation) {
        params.listenerPoint = (0, _utils.coordsToPoints)({
          latitude: listenerLocation.latitude,
          longitude: listenerLocation.longitude
        });
      }

      this.mixParams = _objectSpread(_objectSpread({}, this.mixParams), params);

      if (this.playlist) {
        this.playlist.updateParams(params);
      }

      if (Array.isArray(this.speakerTracks)) {
        this.speakerTracks.forEach(function (t) {
          return t.updateParams(_this.playing, {
            listenerPoint: params.listenerPoint
          });
        });
      }
    }
    /**
     * @param  {number} trackId
     */

  }, {
    key: "skipTrack",
    value: function skipTrack(trackId) {
      if (this.playlist) this.playlist.skip(trackId);
    }
  }, {
    key: "skip",
    value: function skip() {}
    /**
     * @param  {number} trackId
     */

  }, {
    key: "replayTrack",
    value: function replayTrack(trackId) {
      if (this.playlist) this.playlist.replay(trackId);
    }
    /**
     * @returns string
     */

  }, {
    key: "toString",
    value: function toString() {
      return "Roundware Mixer";
    }
    /**
     * Builds speaker tracks & playlist instance if it doesn't exist yet
     */

  }, {
    key: "initContext",
    value: function initContext() {
      if (!this.playlist) {
        if (!this.mixParams.listenerPoint) return console.error("[mixer] listenerPoint was missing while initiating mixer!");
        var listenerPoint = this.mixParams.listenerPoint;
        var selectTrackId = (0, _utils.getUrlParam)(this._windowScope.location.toString(), "rwfSelectTrackId");

        var audioTracks = this._client.audiotracks();

        if (selectTrackId) {
          selectTrackId = Number(selectTrackId);
          audioTracks = audioTracks.filter(function (t) {
            return t.id === selectTrackId;
          });
          console.info("isolating track #".concat(selectTrackId));
        }

        this.playlist = new _playlist.Playlist({
          client: this._client,
          audioTracks: audioTracks,
          listenerPoint: listenerPoint,
          assetPool: this.assetPool,
          audioContext: this.audioContext,
          windowScope: this._windowScope
        });
        this.initializeSpeakers();
        this.updateParams(this.mixParams);
        console.info("Mixer Activated");
      }
    }
    /**
     * @param  {boolean} true to play false to stop
     * @returns boolean
     */

  }, {
    key: "toggle",
    value: function toggle(play) {
      if (typeof play == "boolean") {
        // do based on what asked..
        play ? this.play() : this.stop();
      } else {
        // automatically decide based on playing status
        this.playing ? this.stop() : this.play();
      }

      return this.playing;
    }
  }, {
    key: "play",
    value: function play() {
      this.initContext(); // console.log(`playing`);
      // consssole.log(this.audioContext.currentTime);

      if (this.playing === false) {
        var _this$_client$events;

        (_this$_client$events = this._client.events) === null || _this$_client$events === void 0 ? void 0 : _this$_client$events.logEvent("play_stream");
      }

      this.playing = true;
      if (this.playlist) this.playlist.play();

      if (Array.isArray(this.speakerTracks)) {
        this.speakerTracks.forEach(function (s) {
          s.player.timerStart();
          s.play();
        });
      }
    }
  }, {
    key: "stop",
    value: function stop() {
      this.initContext();

      if (this.playing === true) {
        var _this$_client$events2;

        (_this$_client$events2 = this._client.events) === null || _this$_client$events2 === void 0 ? void 0 : _this$_client$events2.logEvent("pause_stream");
      }

      this.playing = false;
      if (this.playlist) this.playlist.pause();
      if (Array.isArray(this.speakerTracks)) this.speakerTracks.forEach(function (s) {
        s.player.timerStop();
        s.pause();
      });
    }
  }, {
    key: "handleSpeakerEnd",
    value: function handleSpeakerEnd() {
      var _this$speakerTracks, _this$speakerTracks2;

      this.endedSpeakersLength += 1;
      console.log("some speaker ended", this.endedSpeakersLength, (_this$speakerTracks = this.speakerTracks) === null || _this$speakerTracks === void 0 ? void 0 : _this$speakerTracks.length);

      if (this.endedSpeakersLength == ((_this$speakerTracks2 = this.speakerTracks) === null || _this$speakerTracks2 === void 0 ? void 0 : _this$speakerTracks2.length)) {
        this.allSpeakersEndCallback();
      }
    }
  }, {
    key: "replay",
    value: function replay() {
      var _this$speakerTracks3;

      this.endedSpeakersLength = 0;
      var that = this;
      (_this$speakerTracks3 = this.speakerTracks) === null || _this$speakerTracks3 === void 0 ? void 0 : _this$speakerTracks3.forEach(function (s) {
        s.player.pause();
        s.player.replay();
        that.play();
      });
    }
  }, {
    key: "initializeSpeakers",
    value: function initializeSpeakers() {
      this.endedSpeakersLength = 0;

      var speakers = this._client.speakers();

      var that = this;
      this.speakerTracks = speakers.map(function (speakerData) {
        return new _speaker_track.SpeakerTrack({
          audioContext: that.audioContext,
          listenerPoint: that.mixParams.listenerPoint,
          data: speakerData,
          config: that.mixParams.speakerConfig || {
            sync: false,
            length: 600,
            loop: false,
            prefetch: false
          },
          mixer: that
        });
      });
      this.speakerTracks.forEach(function (s) {
        return s.player.onEnd(function () {
          return that.handleSpeakerEnd();
        });
      });
      this.updateParams(this.mixParams);
    }
  }, {
    key: "onAllSpeakersEnd",
    value: function onAllSpeakersEnd(callback) {
      this.allSpeakersEndCallback = callback;
    }
  }]);

  return Mixer;
}();

exports.Mixer = Mixer;