import { Coord, Feature, LineString, MultiPolygon, Point, Polygon } from "@turf/helpers";
import { IAudioContext } from "standardized-audio-context";
import { ISpeakerData, ISpeakerPlayer } from "./types/speaker";
import { SpeakerConfig } from "./types/roundware";
import { Mixer } from "./mixer";
/** A Roundware speaker under the control of the client-side mixer, representing 'A polygonal geographic zone within which an ambient audio stream broadcasts continuously to listeners.
 * Speakers can overlap, causing their audio to be mixed together accordingly.  Volume attenuation happens linearly over a specified distance from the edge of the Speaker’s defined zone.'
 * (quoted from https://github.com/loafofpiecrust/roundware-ios-framework-v2/blob/client-mixing/RWFramework/RWFramework/Playlist/Speaker.swift)
 * */
export declare class SpeakerTrack {
    speakerId: number;
    maxVolume: number;
    minVolume: number;
    attenuationDistanceKm: number;
    uri: string;
    listenerPoint: Point;
    attenuationBorderPolygon?: Feature<MultiPolygon | Polygon>;
    attenuationBorderLineString?: LineString;
    outerBoundary?: Feature<MultiPolygon | Polygon>;
    currentVolume: number;
    speakerData: ISpeakerData;
    soundId: number | undefined;
    player: ISpeakerPlayer;
    audioContext: IAudioContext;
    config: SpeakerConfig;
    mixer: Mixer;
    constructor({ audioContext, listenerPoint, data, config, mixer, }: {
        audioContext: IAudioContext;
        listenerPoint: Feature<Point>;
        mixer: Mixer;
        data: ISpeakerData;
        config: SpeakerConfig;
    });
    outerBoundaryContains(point: Coord): boolean | undefined;
    attenuationShapeContains(point: Coord): boolean | undefined;
    attenuationRatio(atPoint: Coord): number;
    log(string: string): void;
    calculateVolume(): number;
    updateParams(isPlaying: boolean, opts: {
        listenerPoint?: Feature<Point>;
    }): void;
    /**
     * Updates / (Schedules a update to if not playing) volume to match speaker track configurations using the `fade()` function
     * @memberof SpeakerTrack
     */
    updateVolume(): number;
    get logline(): string;
    play(): void;
    pause(): void;
    initPlayer(): void;
    toString(): string;
}
